@@ -22,6 +22,7 @@
 #include "../data_structure/field.h"
 #include "../data_structure/relationship.h"
 #include "../data_structure/glomconversions.h"
+#include "dialog_layout_details.h"
 #include <sstream> //For stringstream
 #include <libintl.h>
 
@@ -38,7 +39,19 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 {
   m_layout_name = "details";
 
-  m_FlowTable.set_columns_count(2);
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout_details"); //TODO: Use a generic layout dialog?
+  if(refXml)
+  {
+    Dialog_Layout_Details* dialog = 0;
+    refXml->get_widget_derived("window_data_layout_details", dialog);
+    if(dialog)
+    {
+      m_pDialogLayout = dialog;
+      m_pDialogLayout->signal_hide().connect( sigc::mem_fun(*this, &Box_Data::on_dialog_layout_hide) );
+    }
+  }
+  
+  m_FlowTable.set_columns_count(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
   m_FlowTable.set_padding(6);
   
   m_strHint = gettext("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
@@ -127,49 +140,12 @@ void Box_Data_Details::fill_from_database_layout()
                           
   Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
   if(document)
-  {          
-    Document_Glom::type_mapLayoutGroupSequence layout_groups = document->get_data_layout_groups_plus_new_fields("details", m_strTableName);
+  {
+    //This map of layout groups will also contain the field information from the database:
+    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
-      const LayoutGroup& group = iter->second;
-  
-      if(group.m_group_name != "others")
-      {
-        FlowTableWithFields::type_map_field_sequence fields;
-
-        //Get the fields:
-        guint sequence = 0;
-        for(LayoutGroup::type_map_items::const_iterator iterItems = group.m_map_items.begin(); iterItems != group.m_map_items.end(); ++iterItems)
-        {
-          Field field;
-          bool found = get_fields_for_table_one_field(m_strTableName, iterItems->second.m_field_name, field);
-          if(found)
-          {
-            fields[sequence] = field;
-            ++sequence;
-          }
-        }
-            
-        m_FlowTable.add_group(group.m_group_name, group.m_title, fields);
-      }
-      else
-      {
-        //Add the extra fields:
-        for(LayoutGroup::type_map_items::const_iterator iterItems = group.m_map_items.begin(); iterItems != group.m_map_items.end(); ++iterItems)
-        {
-          Field field;
-          bool found = get_fields_for_table_one_field(m_strTableName, iterItems->second.m_field_name, field);
-          if(found)
-          {
-            m_FlowTable.add_field(field); //This could add it to a sub-flowtable, or the main flow-table.
-            EntryGlom* pEntry = m_FlowTable.get_field(field);
-
-            //Do not allow editing of auto-increment fields:
-            if(field.get_field_info().get_auto_increment())
-              pEntry->set_editable(false);
-          }
-        }
-      }
+      m_FlowTable.add_layout_group(iter->second);
     }
   }
       
@@ -218,21 +194,19 @@ void Box_Data_Details::fill_from_database()
           if(pDoc)
           {
             //Get glom-specific field info:
-            Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
+            //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
 
             const int row_number = 0; //The only row.
             const int cols_count = result->get_n_columns();
-            for(int i = 0; i < cols_count; i ++)
+            for(int i = 0; i < cols_count; ++i)
             {
-              const Field field = m_Fields[i];
-                
-              EntryGlom* pEntry = m_FlowTable.get_field(field);        
-              if(pEntry)
-              {
-                //Field value:
-                Gnome::Gda::Value value = result->get_value_at(i, row_number);
-                pEntry->set_value(value);
-              }
+              const Field& field = m_Fields[i];
+
+              //Field value:
+              Gnome::Gda::Value value = result->get_value_at(i, row_number);
+              g_warning("debug 1");
+              m_FlowTable.set_field_value(field, value);
+                g_warning("debug 2");
             }
           }
         }
@@ -312,28 +286,24 @@ void Box_Data_Details::fill_related()
            add_view(pBox); //So that it knows about the Document.
            m_Notebook_Related.pages().push_back( Gtk::Notebook_Helpers::TabElem(*pBox, relationship.get_name()) );
               
-           EntryGlom* pEntry = m_FlowTable.get_field(from_field);
-           if(pEntry)
+           Gnome::Gda::Value value = m_FlowTable.get_field_value(from_field);
+
+           Field field;
+           bool test = get_fields_for_table_one_field(m_strTableName, from_field, field);
+           if(test)
            {
-             Gnome::Gda::Value value = pEntry->get_value();
-
-             Field field;
-             bool test = get_fields_for_table_one_field(m_strTableName, from_field, field);
-             if(test)
-             {
-               pBox->init_db_details(get_database_name(), relationship, value, m_primary_key_value);
-               pBox->show_all();
+             pBox->init_db_details(get_database_name(), relationship, value, m_primary_key_value);
+             pBox->show_all();
 
 
-               //Connect signals:
-               pBox->signal_record_added.connect( sigc::bind(
-                 sigc::mem_fun(*this, &Box_Data_Details::on_related_record_added), relationship.get_from_field() )
-               );
+             //Connect signals:
+             pBox->signal_record_added.connect( sigc::bind(
+               sigc::mem_fun(*this, &Box_Data_Details::on_related_record_added), relationship.get_from_field() )
+             );
 
-               pBox->signal_user_requested_details().connect( sigc::bind(
-                 sigc::mem_fun(*this, &Box_Data_Details::on_related_user_requested_details), relationship.get_to_table() )
-               );
-             }
+             pBox->signal_user_requested_details().connect( sigc::bind(
+               sigc::mem_fun(*this, &Box_Data_Details::on_related_user_requested_details), relationship.get_to_table() )
+             );
            }
          }
        }
@@ -362,7 +332,7 @@ void Box_Data_Details::on_button_new()
         Gnome::Gda::Value primary_key_value = GlomConversions::parse_value(field_primary_key.get_glom_type(), util_string_from_decimal(generated_id), parsed);
         if(parsed)
         {
-          record_new(primary_key_value);
+          record_new(false /* use entered field data */, primary_key_value);
           init_db_details(primary_key_value);
         }
       }
@@ -424,11 +394,12 @@ void Box_Data_Details::on_button_nav_last()
 
 Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const Field& field) const
 {
-  const EntryGlom* entry = m_FlowTable.get_field(field);
-  if(entry)
-    return entry->get_value();
-  else
-    return Gnome::Gda::Value(); //null.
+  return m_FlowTable.get_field_value(field);
+}
+
+void Box_Data_Details::set_entered_field_data(const Field& field, const Gnome::Gda::Value& value)
+{
+  m_FlowTable.set_field_value(field, value);
 }
 
 Gnome::Gda::Value Box_Data_Details::get_primary_key_value_selected()
@@ -470,7 +441,7 @@ void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue
   {
     //Set the From key value, to link the new related record (the first one so far) with the parent record.
     m_AddDel.set_value(iKey, m_ColumnValue, strKeyValue);
-    on_AddDel_user_changed(iKey, m_ColumnValue); //Update the database.
+    on_adddel_user_changed(iKey, m_ColumnValue); //Update the database.
   }
   */
 
@@ -508,111 +479,109 @@ Box_Data_Details::type_signal_user_requested_related_details Box_Data_Details::s
   return m_signal_user_requested_related_details;
 }
 
-void Box_Data_Details::on_flowtable_field_edited(Glib::ustring id)
+void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
     return;
     
-  EntryGlom* entry = m_FlowTable.get_field(id);
-  if(entry)
-  {
-     const Glib::ustring strPrimaryKey_Name = get_primarykey_name();
-     const Glib::ustring strFieldName = id;
-     const Gnome::Gda::Value field_value = entry->get_value();
+   const Glib::ustring strPrimaryKey_Name = get_primarykey_name();
+   const Glib::ustring strFieldName = id;
 
-     if(!strPrimaryKey_Name.empty())
+   if(!strPrimaryKey_Name.empty())
+   {
+     Field fieldInfoPK;
+     bool test = get_field(strPrimaryKey_Name, fieldInfoPK);
+     if(!test)
+       g_warning("Box_Data_Details::on_flowtable_field_edited(): field not found");
+
+     Gnome::Gda::Value primary_key_value = get_primary_key_value();
+     if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a primary key value:
      {
-       Field fieldInfoPK;
-       bool test = get_field(strPrimaryKey_Name, fieldInfoPK);
+       Field field;
+       bool test = get_field(strFieldName, field);
        if(!test)
-         g_warning("Box_Data_Details::on_flowtable_field_edited(): field not found");
+         g_warning("Box_Data_Details::on_flowtable_field_edited: field not found");
 
-       Gnome::Gda::Value primary_key_value = get_primary_key_value();
-       if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a primary key value:
+       //Update the field in the record (the record with this primary key):
+       try
        {
-         Field field;
-         bool test = get_field(strFieldName, field);
-         if(!test)
-           g_warning("Box_Data_Details::on_flowtable_field_edited: field not found");
-           
-         //Update the field in the record (the record with this primary key):
-         try
+         Glib::ustring strQuery = "UPDATE " + m_strTableName;
+         strQuery += " SET " +  /* get_table_name() + "." +*/ strFieldName + " = " + field.sql(field_value);
+         strQuery += " WHERE " + get_table_name() + "." + strPrimaryKey_Name + " = " + fieldInfoPK.sql(get_primary_key_value());
+         bool bTest = Query_execute(strQuery);
+
+         if(!bTest)
          {
-           Glib::ustring strQuery = "UPDATE " + m_strTableName;
-           strQuery += " SET " +  /* get_table_name() + "." +*/ strFieldName + " = " + field.sql(field_value);
-           strQuery += " WHERE " + get_table_name() + "." + strPrimaryKey_Name + " = " + fieldInfoPK.sql(get_primary_key_value());
-           bool bTest = Query_execute(strQuery);
-
-           if(!bTest)
-           {
-             //Update failed.
-             fill_from_database(); //Replace with correct values.
-           }
-           else
-           {
-             //Get-and-set values for lookup fields, if this field triggers those relationships:
-             do_lookups(field, field_value, fieldInfoPK, primary_key_value);
-             
-             //If this is a foreign key then refresh the related records:
-             bool bIsForeignKey = false;
-             Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
-             for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
-             {
-               const Relationship& relationship = *iter;
-
-               if(relationship.get_from_field() == strFieldName)
-               {
-                 bIsForeignKey = true;
-                 break;
-               }
-             }
-
-             if(bIsForeignKey)
-               fill_related();
-           }
-
+           //Update failed.
+           fill_from_database(); //Replace with correct values.
          }
-         catch(const std::exception& ex)
+         else
          {
-           handle_error(ex);
+           //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
+           m_FlowTable.set_field_value(field, field_value);
+           
+           //Get-and-set values for lookup fields, if this field triggers those relationships:
+           do_lookups(field, field_value, fieldInfoPK, primary_key_value);
+
+           //If this is a foreign key then refresh the related records:
+           bool bIsForeignKey = false;
+           Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
+           for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
+           {
+             const Relationship& relationship = *iter;
+
+             if(relationship.get_from_field() == strFieldName)
+             {
+               bIsForeignKey = true;
+               break;
+             }
+           }
+
+           if(bIsForeignKey)
+             fill_related();
+         }
+
+       }
+       catch(const std::exception& ex)
+       {
+         handle_error(ex);
+       }
+     }
+     else
+     {
+       //There is no current primary key:
+
+       if(fieldInfoPK.get_field_info().get_auto_increment()) //If the primary key is an auto-increment:
+       {
+         if(strFieldName == strPrimaryKey_Name) //If edited field is the primary key.
+         {
+           //Warn user that they can't choose their own primary key:
+           Gtk::MessageDialog dialog ("The primary key is auto-incremented.\n You may not enter your own primary key value.");
+           dialog.run();
          }
        }
        else
        {
-         //There is no current primary key:
+         //It is not auto-generated:
 
-         if(fieldInfoPK.get_field_info().get_auto_increment()) //If the primary key is an auto-increment:
+         if(strFieldName == strPrimaryKey_Name) //if it is the primary key that' being edited.
          {
-           if(strFieldName == strPrimaryKey_Name) //If edited field is the primary key.
-           {
-             //Warn user that they can't choose their own primary key:
-             Gtk::MessageDialog dialog ("The primary key is auto-incremented.\n You may not enter your own primary key value.");
-             dialog.run();
-           }
+           //Create new record with this primary key,
+           //and all the other field values too.
+           //see comments after 'else':
+           record_new(true /* use entered field data */);
          }
          else
          {
-           //It is not auto-generated:
+           //The record does not exist yet.
+           //The values in the other fields will have to wait
+           //until the primary key is set by the user.
 
-           if(strFieldName == strPrimaryKey_Name) //if it is the primary key that' being edited.
-           {
-             //Create new record with this primary key,
-             //and all the other field values too.
-             //see comments after 'else':
-             record_new_from_entered();
-           }
-           else
-           {
-             //The record does not exist yet.
-             //The values in the other fields will have to wait
-             //until the primary key is set by the user.
-
-             set_unstored_data(true); //Cause a warning if this is never put into the database.
-           }
+           set_unstored_data(true); //Cause a warning if this is never put into the database.
          }
-       } //if(get_primary_key_value().size())
-     } //if(strPrimaryKey_Name.size())
-  }
+       }
+     } //if(get_primary_key_value().size())
+   } //if(strPrimaryKey_Name.size())
 }
 
 void Box_Data_Details::do_lookups(const Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
@@ -634,9 +603,7 @@ void Box_Data_Details::do_lookups(const Field& field_changed, const Gnome::Gda::
        Gnome::Gda::Value value = get_lookup_value(iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
 
        //Add it to the view:
-       EntryGlom* entry = m_FlowTable.get_field(field_lookup_name);
-       if(entry)
-         entry->set_value(value);
+       m_FlowTable.set_field_value(field_lookup_name, value);
 
        //Add it to the database (even if it is not shown in the view)
        Glib::ustring strQuery = "UPDATE " + m_strTableName;
